%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing correction factors      %
%          for the LR trace test                                 %
%                                                                %
%          Function: mlcointbasic                                %
%                                                                %
%          Input:                                                %
%          	            S_00: nxn matrix                         %
%          	            S_01: nxnq1 matrix                       %
%                       S_10: nq1xn matrix                       %
%                       S_11: nq1xnq1 matrix                     %
%                       M_02: nxn(p-1)+d matrix                  %
%                       M_22: n(p-1)+dxn(p-1)+d matrix           %
%                       M_12: nq1xn(p-1)+d matrix                %
%                       beta: nq1xr matrix                       %
%                       d: number of deterministic variables     %
%                       p: number of lags                        %
%                       T: number of observations                %
%                       deterselect: 1x4 0-1 vector indicating   %
%                           the deterministic model              %
%                       mywait: handle for waiting dialog        %
%                                                                %
%          Output:                                               %
%                       corrfact: nx1 vector with correction     %
%                           factors                              %
%                                                                %
%          This version:        June 1, 2004                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function corrfact = barttr(S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,d,p,T,deterselect,waitf)

%
% Preliminaries
%

if nargin<13;
   waitf = [];
end;

n = size(S_00,1);

corrfact = [];
for r=0:n-1;
   k = n-r;
   %
   % manipulating waiting dialogue
   %
   if isempty(waitf)==0;
      if n*p>15;
         txt = strvcat(' ',['Computing trace test Bartlett correction factor for ' num2str(k) ' common trends. Please hold!'],' ');
         set(waitf.text,'String',txt);
      end;
   end;
   %
   % Setting up the basic parameters for given rank
   %
   if r==0;
      thisbeta = [];
      alpha = [];
      Omega = S_00;
      alpha_orth = eye(n);
      if isempty(M_22)==0;
         Psi = M_02/M_22;
      else;
         Psi = [];
      end;
   else;
      thisbeta = beta(:,1:r);
      Omega = S_00-(((S_01*thisbeta)/(thisbeta'*S_11*thisbeta))*thisbeta'*S_10);
      alpha = (S_01*thisbeta)/(thisbeta'*S_11*thisbeta);
      alpha_orth = null(alpha');
      if isempty(M_22)==0;
         Psi = (M_02/M_22)-(alpha*thisbeta'*(M_12/M_22));
      else;
         Psi = [];
      end;
   end;
   %
   % Coefficients on lags
   %
   if p>=2;
      Phi = Psi(:,d+1:d+n*(p-1));
   end;
   %
   % in VAR(1) form
   %
   if r>0;
      thisbeta = thisbeta(1:n,:);
      if p>=2;
         P = [[(thisbeta'*alpha)+eye(r) thisbeta'*Phi];[alpha Phi]];
         Q = [thisbeta';eye(n)];
         if p>=3;
            P = [P;[zeros((p-2)*n,r) eye((p-2)*n) zeros((p-2)*n,n)]];
            Q = [Q;zeros((p-2)*n,n)];
         end;
      else;
         P = [[(thisbeta'*alpha)+eye(r) zeros(r,n)];[alpha zeros(n,n)]];
         Q = [thisbeta';eye(n)];
      end;
   else;
      if p>=2;
         Q = [thisbeta';eye(n)];
         P = Phi;
         if p>=3;
            Q = [Q;zeros((p-2)*n,n)];
            P = [P;[eye((p-2)*n) zeros((p-2)*n,n)]];
         end;
      else;
         Q = eye(n);
         P = zeros(n,n);
      end;
   end;
   %
   % Dealing with variances
   %
   OmegaT = Q*Omega*Q';
   n_y = size(P,1);
   I_PP = eye(n_y*n_y)-kron(P,P);
   I_PP_inv = I_PP\eye(n_y*n_y);
   Sigma = I_PP_inv*vec(OmegaT);
   Sigma = vecinv(Sigma,n_y);
   Sigma_inv = Sigma\eye(n_y);
   I_P = eye(n_y)-P;
   I_P_inv = I_P\eye(n_y);
   IP_inv = (eye(n_y)+P)\eye(n_y);
   I_P_prime_inv = (eye(n_y)-P')\eye(n_y);
   %
   % Computing the Choleski decomposition
   % of alpha_orth'*Omega*alpha_orth and
   % making sure it's lower triangular
   %
   H = chol(alpha_orth'*Omega*alpha_orth)';
   %
   % more variances
   %
   inv_H = eye(size(H,1))/H;
   psi = I_P_inv*Q*Omega*alpha_orth*inv_H';
   V_psi = psi*psi'*Sigma_inv;
   %
   % constructing c_1
   %
   c_1 = sum(diag(I_P_inv*Q*((Omega*alpha_orth)/(alpha_orth'*Omega*alpha_orth))*alpha_orth'*Omega*Q'*I_P_prime_inv*Sigma_inv));
   %
   % constructing c_2
   %
   c_2 = sum(diag(eye(n_y)-(I_P_inv*OmegaT*I_P_prime_inv*Sigma_inv)));
   %
   % constructing c_3
   %
   c_31 = sum(diag(kron(I_P*V_psi,P)*I_PP_inv));
   c_32 = sum(diag(V_psi*P*IP_inv));
   c_3 = c_31+c_32;
   %
   % Constructing gamma(k,n_d) approximation
   %
   if deterselect(1,1)==1;
      gamma_k = -1.469+(1.457*(k/T))-(0.611*(k/T)*(k/T));
      n_d = 1;
   elseif deterselect(1,2)==1;
      gamma_k = -0.486+(0.18*(k/T))-(0.016*(k/T)*(k/T));
      n_d = 0;
   elseif deterselect(1,3)==1;
      gamma_k = -2.338+(3.306*(k/T))-(1.702*(k/T)*(k/T));
      n_d = 2;
   else;
      gamma_k = -1.469+(1.457*(k/T))-(0.611*(k/T)*(k/T));
      n_d = 1;
   end;
   %
   % Constructing a(T,k,n_d) approximation
   %
   a_k = 1+(0.45*(k/T))+(1.2*(k/T)*(k/T));
   %
   % Constructing b(k,T)
   %
   b_k = c_1+((k*c_2+2*(c_3-(n_d*c_1)))*(gamma_k/(k*k)));
   if a_k*(1+(b_k/T))>0;
      corrfact = [corrfact;a_k*(1+(b_k/T))];
   else;
      corrfact = [corrfact;1];
   end;
end;
%
% manipulating waiting dialogue
%
if isempty(waitf)==0;
   if n*p>15;
      txt = strvcat(' ',' ','Initializing... Please hold!',' ');
      set(waitf.text,'String',txt);
   end;
end;
%
% end of barttr.m
%
