function h = editbox(varargin)
%
% An editable text controls which uses MatLab's default edit context menu for
% text objects.
%
% Properties
%    String: Text string (default is '')
%    Position: 1x4 vector with left bottom width height values (default is [0 0 10 10])
%    Enable: on or off (default is on)
%    BackgroundColor: 1x3 vector with color values (default is [1 1 1])
%    Color: Text color (default for text objects)
%    FontStyle, FontAngle, FontName, FontUnits, FontWeight (default for text objects)
%    HorizontalAlignment (default for text objects)
%    VerticalAlignment (default is bottom)
%    Visible: on or off (on is default)
%    Tag: string (default is '')
%    Parent: figure (default is gcf)
%    Units are always pixels (...)
%    Callback: function that runs once editing has finished (default is empty string)
%              Maximum number of inputs to the callback function is 10. No outputs
%              allowed. Inputs can be strings or numbers.
%
% To change properties:
%    Use editbox(handle,'PropertyName',PropertyValue,...) always works
%
%    set and get works for all properties but:
%                   enable, position, visible, backgroundcolor, and callback
%
% The function imposes a WindowButtonMotionFcn called 'editbox(2)'.
%
% To Do: make the text string never exceed the size of the position rectangle
%

if isnumeric(varargin{1})==0;
   %
   % create an axesbox first
   %
   mypos = [0 0 10 10];
   num = round(size(varargin,2)/2);
   for i=1:num;
      k = 1+2*(i-1);
      if ischar(varargin{k})==1;
         if strcmp(lower(varargin{k}),'position')==1;
            mypos = varargin{k+1};
         end;
      end;
   end;
   bg_color = get(0,'defaultuicontrolbackgroundcolor');
   bg_color = mean(bg_color);
   %
   % back ground axes
   %
   bg_axes = axes('Units','pixels', ...
      'Position',mypos+[2 2 -2 -2], ...
      'Tag','bg_axes', ...
      'ButtonDownFcn','editbox(1)', ...
      'Color',[1 1 1], ...
      'XColor',[1 1 1], ...
      'YColor',[1 1 1], ...
      'Box','on', ...
      'Visible','off', ...
      'TickLength',[0 0], ...
      'XTickLabel',[], ...
      'YTickLabel',[], ...
      'XTick',[0 1], ...
      'YTick',[0 1]);
   %
   % and now the text handle
   %
   curr_ini = find_inisettings;
   text_handle = text('Parent',bg_axes, ...
      'Interpreter','none', ...
      'Units','pixels', ...
      'Clipping','on', ...
      'VerticalAlignment','bottom', ...
      'HorizontalAlignment','left', ...
      'Position',[5 5], ...
      'String','', ...
      'Editing','off', ...
      'FontSize',curr_ini.gui_fontsize, ...
      'FontName',curr_ini.gui_fontname, ...
      'FontWeight',curr_ini.gui_fontweight, ...
      'FontAngle',curr_ini.gui_fontangle, ...
      'ButtonDownFcn','editbox(1)', ...
      'Color',[0 0 0], ...
      'Visible','off', ...
      'UserData',[], ...
      'Tag','');
   %
   % handles for the box describing the borders of the control
   %
   ax_handles = axesbox(mypos,'',0,[0.5 0 bg_color 1],'on');
   set(ax_handles,'Visible','off','Layer','top');
   %
   % reset controls
   %
   prop.callback = '';
   prop.units = 'pixels';
   prop.parent = gcf;
   prop.axhandles = ax_handles;
   prop.enable = 'on';
   prop.handle = text_handle;
   prop.horizontalalignment = 'left';
   prop.string = '';
   prop.color = [0 0 0];
   prop.visible = 'on';
   prop.userdata = [];
   prop.tag = '';
   prop.fontsize = 7;
   prop.fontname = get(text_handle,'FontName');
   prop.fontunits = get(text_handle,'FontUnits');
   prop.fontangle = get(text_handle,'FontAngle');
   prop.fontweight = get(text_handle,'FontWeight');
   prop.backgroundcolor = [1 1 1];
   prop.position = mypos;
   prop.buttondownfcn = 'editbox(1)';
   for i=1:num;
      k = 1+2*(i-1);
      if ischar(varargin{k})==1;
         if strcmp(lower(varargin{k}),'string')==1;
            set(text_handle,'String',varargin{k+1});
            prop.string = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'backgroundcolor')==1;
            set(bg_axes,'Color',varargin{k+1},'XColor',varargin{k+1},'YColor',varargin{k+1});
            prop.backgroundcolor = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'parent')==1;
            if (ishandle(varargin{k+1})==1)&(strcmp(get(varargin{k+1},'Type'),'figure')==1);
               set(bg_axes,'Parent',varargin{k+1});
               set(ax_handles,'Parent',varargin{k+1});
               prop.parent = varargin{k+1};
            end;
         elseif strcmp(lower(varargin{k}),'visible')==1;
            prop.visible = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'enable')==1;
            set(text_handle,'UserData',varargin{k+1});
            prop.enable = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'color')==1;
            set(text_handle,'Color',varargin{k+1});
            prop.color = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'fontname')==1;
            set(text_handle,'FontName',varargin{k+1});
            prop.fontname = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'fontsize')==1;
            set(text_handle,'FontSize',varargin{k+1});
            prop.fontsize = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'fontunits')==1;
            set(text_handle,'FontUnits',varargin{k+1});
            prop.units = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'fontangle')==1;
            set(text_handle,'FontAngle',varargin{k+1});
            prop.fontangle = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'fontweight')==1;
            set(text_handle,'FontWeight',varargin{k+1});
            prop.fontweight = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'horizontalalignment')==1;
            set(text_handle,'HorizontalAlignment',varargin{k+1});
            prop.horizontalalignment = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'tag')==1;
            set(text_handle,'Tag',varargin{k+1});
            prop.tag = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'userdata')==1;
            set(text_handle,'UserData',varargin{k+1});
            prop.userdata = varargin{k+1};
         elseif strcmp(lower(varargin{k}),'callback')==1;
            prop.callback = strtrim(varargin{k+1});
         end;
      end;
   end;
   %
   % change color of text if enable is off
   %
   if strcmp(prop.enable,'off')==1;
      set(text_handle,'Color',[0.4 0.4 0.4],'ButtonDownFcn','');
      prop.buttondownfcn = '';
   end;
   %
   % show or hide
   %
   set(bg_axes,'Visible',prop.visible);
   set(ax_handles,'Visible',prop.visible);
   set(text_handle,'Visible',prop.visible);
   %
   set(bg_axes,'UserData',prop,'ButtonDownFcn',prop.buttondownfcn);
   if (strcmp(get(prop.parent,'WindowButtonMotionFcn'),'')==1)|(strcmp(get(prop.parent,'WindowButtonMotionFcn'),'editbox(2)')==0);
      set(prop.parent,'WindowButtonMotionFcn','editbox(2)');
   end;
elseif (ishandle(varargin{1})==1)&(size(varargin,2)>1);
   %
   % if nargin is odd then set properties, else get properties
   %
   text_handle = varargin{1};
   bg_axes = get(text_handle,'Parent');
   prop = get(bg_axes,'UserData');
   if isodd(size(varargin,2))==1;
      %
      % reset control, e.g. enable, etc.
      %
      ax_handles = prop.axhandles;
      num = round(size(varargin,2)/2)-1;
      for i=1:num;
         k = 2*i;
         if ischar(varargin{k})==1;
            if strcmp(lower(varargin{k}),'string')==1;
               set(text_handle,'String',varargin{k+1});
               prop.string = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'backgroundcolor')==1;
               set(bg_axes,'Color',varargin{k+1},'XColor',varargin{k+1},'YColor',varargin{k+1});
               prop.backgroundcolor = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'parent')==1;
               if (ishandle(varargin{k+1})==1)&(strcmp(get(varargin{k+1},'Type'),'figure')==1);
                  set(bg_axes,'Parent',varargin{k+1});
                  set(ax_handles,'Parent',varargin{k+1});
                  prop.parent = varargin{k+1};
               end;
            elseif strcmp(lower(varargin{k}),'visible')==1;
               prop.visible = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'enable')==1;
               set(text_handle,'UserData',varargin{k+1});
               prop.enable = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'color')==1;
               set(text_handle,'Color',varargin{k+1});
               prop.color = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'fontname')==1;
               set(text_handle,'FontName',varargin{k+1});
               prop.fontname = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'fontsize')==1;
               set(text_handle,'FontSize',varargin{k+1});
               prop.fontsize = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'fontunits')==1;
               set(text_handle,'FontUnits',varargin{k+1});
               prop.units = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'fontangle')==1;
               set(text_handle,'FontAngle',varargin{k+1});
               prop.fontangle = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'fontweight')==1;
               set(text_handle,'FontWeight',varargin{k+1});
               prop.fontweight = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'horizontalalignment')==1;
               set(text_handle,'HorizontalAlignment',varargin{k+1});
               prop.horizontalalignment = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'tag')==1;
               set(text_handle,'Tag',varargin{k+1});
               prop.tag = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'userdata')==1;
               set(text_handle,'UserData',varargin{k+1});
               prop.userdata = varargin{k+1};
            elseif strcmp(lower(varargin{k}),'callback')==1;
               prop.callback = strtrim(varargin{k+1});
            end;
         end;
      end;
      %
      % change color of text if enable is off
      %
      if strcmp(prop.enable,'off')==1;
         set(text_handle,'Color',[0.4 0.4 0.4],'ButtonDownFcn','');
         prop.buttondownfcn = '';
      else;
         prop.buttondownfcn = 'editbox(1)';
         set(text_handle,'Color',prop.color,'ButtonDownFcn','editbox(1)');
      end;
      %
      % show or hide
      %
      set(bg_axes,'Visible',prop.visible);
      set(ax_handles,'Visible',prop.visible);
      set(text_handle,'Visible',prop.visible);
      %
      set(bg_axes,'UserData',prop,'ButtonDownFcn',prop.buttondownfcn);
   elseif size(varargin,2)==2;
      %
      % get properties
      %
      if ischar(varargin{2})==1;
         propfields = fieldnames(prop);
         for i=1:size(propfields,1);
            if strcmp(propfields{i},lower(varargin{2}))==1;
               text_handle = getfield(prop,propfields{i});
               break;
            end;
         end;
      end;
   end;
elseif varargin{1}==1;
   %
   % buttondown function
   %
   if strcmp(get(gco,'Type'),'text')==1
      par = get(gco,'Parent');
      prop = get(par,'UserData');
      if (isfield(prop,'enable')==1)&(isfield(prop,'handle')==1);
         if strcmp(prop.enable,'on')==1;
            if strcmp(get(prop.handle,'Editing'),'off')==1;
               set(gcf,'Pointer','arrow','WindowButtonMotionFcn','');
               set(prop.handle,'Editing','on');
               waitfor(prop.handle,'Editing','off');
               if ishandle(prop.handle)==1;
                  set(gcf,'WindowButtonMotionFcn','editbox(2)');
                  refresh;
                  new_str = get(prop.handle,'String');
                  tag_str = get(prop.handle,'Tag');
                  if (isempty(new_str)==0)&(strcmp(tag_str,'TeXHeaderEdit')==0);
                     set(prop.handle,'String',strtrim(new_str(1,:)));
                  end;
                  if strcmp(prop.callback,'')==0;
                     in_str = strrep(prop.callback,'(',' ');
                     in_str = strrep(in_str,')',' ');
                     in_str = strrep(in_str,',',' ');
                     in_str = strrep(in_str,'''','');
                     %
                     % replace all spaces within square brackets with _
                     %
                     leftpos = findstr(in_str,'[');
                     rightpos = findstr(in_str,']');
                     if (isempty(leftpos)==0)&(isempty(rightpos)==0);
                        if size(leftpos,2)==size(rightpos,2);
                           spacepos = findstr(in_str,' ');
                           if isempty(spacepos)==0;
                              num = 1;
                              i = 1;
                              while (i<=size(spacepos,2))&(num<=size(leftpos,2));
                                 if (spacepos(1,i)>=leftpos(1,num))&(spacepos(1,i)<=rightpos(1,num));
                                    in_str = [in_str(1,1:spacepos(1,i)-1) '_' in_str(1,spacepos(1,i)+1:size(in_str,2))];
                                    i = i+1;
                                 elseif spacepos(1,i)>rightpos(1,num);
                                    num = num+1;
                                 elseif spacepos(1,i)<leftpos(1,num);
                                    i = i+1;
                                 end;
                              end;
                           end;
                        end;
                     end;
                     in_str = cellstr(vecstr(in_str))';
                     for i=1:size(in_str,2);
                        my_str = in_str{i};
                        my_str = strrep(my_str,'[','');
                        my_str = strrep(my_str,']','');
                        my_str = strrep(my_str,'_',' ');
                        if isempty(mystr2num(my_str))==1;
                           my_struct(i).input = my_str;
                        else;
                           my_struct(i).input = mystr2num(my_str);
                        end;
                     end;
                     if size(in_str,2)==1;
                        feval(my_struct(1).input);
                     elseif size(in_str,2)==2;
                        feval(my_struct(1).input,my_struct(2).input);
                     elseif size(in_str,2)==3;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input);
                     elseif size(in_str,2)==4;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input);
                     elseif size(in_str,2)==5;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input);
                     elseif size(in_str,2)==6;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input);
                     elseif size(in_str,2)==7;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input);
                     elseif size(in_str,2)==8;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input);
                     elseif size(in_str,2)==9;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input);
                     elseif size(in_str,2)==10;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input,my_struct(10).input);
                     elseif size(in_str,2)==11;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input,my_struct(10).input,my_struct(11).input);
                     end;
                  end;
               end;
            end;
         end;
      end;
   elseif strcmp(get(gco,'Tag'),'bg_axes')==1;
      prop = get(gco,'UserData');
      if (isfield(prop,'enable')==1)&(isfield(prop,'handle')==1);
         if strcmp(prop.enable,'on')==1;
            if strcmp(get(prop.handle,'Editing'),'off')==1;
               set(gcf,'Pointer','arrow','WindowButtonMotionFcn','');
               set(prop.handle,'Editing','on');
               waitfor(prop.handle,'Editing','off');
               if ishandle(prop.handle)==1;
                  set(gcf,'WindowButtonMotionFcn','editbox(2)');
                  refresh;
                  new_str = get(prop.handle,'String');
                  tag_str = get(prop.handle,'Tag');
                  if (isempty(new_str)==0)&(strcmp(tag_str,'TeXHeaderEdit')==0);
                     set(prop.handle,'String',strtrim(new_str(1,:)));
                  end;
                  if strcmp(prop.callback,'')==0;
                     in_str = strrep(prop.callback,'(',' ');
                     in_str = strrep(in_str,')',' ');
                     in_str = strrep(in_str,',',' ');
                     in_str = strrep(in_str,'''','');
                     %
                     % replace all spaces within square brackets with _
                     %
                     leftpos = findstr(in_str,'[');
                     rightpos = findstr(in_str,']');
                     if (isempty(leftpos)==0)&(isempty(rightpos)==0);
                        if size(leftpos,2)==size(rightpos,2);
                           spacepos = findstr(in_str,' ');
                           if isempty(spacepos)==0;
                              num = 1;
                              i = 1;
                              while (i<=size(spacepos,2))&(num<=size(leftpos,2));
                                 if (spacepos(1,i)>=leftpos(1,num))&(spacepos(1,i)<=rightpos(1,num));
                                    in_str = [in_str(1,1:spacepos(1,i)-1) '_' in_str(1,spacepos(1,i)+1:size(in_str,2))];
                                    i = i+1;
                                 elseif spacepos(1,i)>rightpos(1,num);
                                    num = num+1;
                                 elseif spacepos(1,i)<leftpos(1,num);
                                    i = i+1;
                                 end;
                              end;
                           end;
                        end;
                     end;
                     in_str = cellstr(vecstr(in_str))';
                     for i=1:size(in_str,2);
                        my_str = in_str{i};
                        my_str = strrep(my_str,'[','');
                        my_str = strrep(my_str,']','');
                        my_str = strrep(my_str,'_',' ');
                        if isempty(mystr2num(my_str))==1;
                           my_struct(i).input = my_str;
                        else;
                           my_struct(i).input = mystr2num(my_str);
                        end;
                     end;
                     if size(in_str,2)==1;
                        feval(my_struct(1).input);
                     elseif size(in_str,2)==2;
                        feval(my_struct(1).input,my_struct(2).input);
                     elseif size(in_str,2)==3;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input);
                     elseif size(in_str,2)==4;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input);
                     elseif size(in_str,2)==5;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input);
                     elseif size(in_str,2)==6;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input);
                     elseif size(in_str,2)==7;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input);
                     elseif size(in_str,2)==8;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input);
                     elseif size(in_str,2)==9;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input);
                     elseif size(in_str,2)==10;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input,my_struct(10).input);
                     elseif size(in_str,2)==11;
                        feval(my_struct(1).input,my_struct(2).input,my_struct(3).input,my_struct(4).input,my_struct(5).input,my_struct(6).input,my_struct(7).input,my_struct(8).input,my_struct(9).input,my_struct(10).input,my_struct(11).input);
                     end;
                  end;
               end;
            end;
         end;
      end;
   end;
elseif varargin{1}==2;
   %
   % window button motion function
   % changes the cursor to ibeam if over text
   % object with enable
   %
   this_handle = overobj_fcn('axes');
   if (isempty(this_handle)==0)&(ishandle(this_handle)==1);
      if strcmp(get(this_handle,'Tag'),'bg_axes')==1;
         set(gcf,'Pointer','ibeam');
      else;
         set(gcf,'Pointer','arrow');
      end;
   else;
      set(gcf,'Pointer','arrow');
   end;
end;

if nargout==1;
   h = text_handle;
end;

%
% end of editbox.m
