function [deletevars,keepvars] = getdelvars(x1,x1_r)

%
% Checks which deterministic variables are affinely dependent. Reports
% these into vector deletevars, and the ones that are affinely independent
% are reported into keepvars.
%
% Example 1:
%
%        x1 = [[1 1 1];[2 1 2];[3 1 3];[4 1 4]]
%        x1_r = [0;1;2;3]
%
%        deletevars = [1;3]
%        keepvars = 2
%
% Example 2:
%        x1 = [[1 1 1];[2 1 2];[3 1 3];[4 1 4]]
%        x1_r = []
%
%        deletevars = 3
%        keepvars = [1;2]
%
% NOTES: The reason for having x1_r is that deterministic variables may be
%        placed in different parts of a VEC model, where one part should
%        always remain (x1_r).
%
%        Formally, the first or the third columns of x1 are not linearly
%        dependent wrt x1_r in Example 1, but by adding e.g. the mean of
%        x1_r and removing the mean of the first column of x1 to the first
%        column of x1, we find that the transformed first column of x1 and
%        x1_r are linearly dependent
%
%        x1_r is optional.
%

if nargin==1;
   x1_r = [];
end;

if (size(x1,1)==size(x1_r,1))|(isempty(x1_r)==1);
   x = [x1_r x1];
else;
   if size(x1,2)==size(x1_r,2);
      x = [x1_r' x1'];
   else;
      [T1 n1] = size(x1);
      [T2 n2] = size(x1_r);
      if n1>T1;
         x1 = x1';
         [T1 n1] = size(x1);
      end;
      if n2>T2;
         x1_r = x1_r';
         [T2 n2] = size(x1_r);
      end;
      if T1>T2;
         x = [x1_r x1(1:T2,:)];
      else;
         x = [x1_r(1:T1,:) x1];
      end;
   end;
end;

[T d1] = size(x);
if d1>T;
   x = x';
   [T d1] = size(x);
end;

deletevars = [];
for i=1:d1-1;
   xi = x(:,i);
   for j=i+1:d1;
      done = 0;
      xj = x(:,j);
      if rank(xj)==1;
         xj = xj-mymean(xj)+mymean(xi);
      end;
      xij = [xi xj];
      if rank(xij'*xij)<2;
         if abs(sum(xi))<1.0e-006;
            kend = size(deletevars,1);
            if kend>=1;
               for k=1:kend;
                  if i==deletevars(k,1);
                     done = 1;
                  end;
               end;
               if done==0;
                  deletevars = [deletevars;i];
               end;
            else;
               deletevars = [deletevars;i];
            end;
         else;
            kend = size(deletevars,1);
            if kend>=1;
               for k=1:kend;
                  if j==deletevars(k,1);
                     done = 1;
                  end;
               end;
               if done==0;
                  deletevars = [deletevars;j];
               end;
            else;
               deletevars = [deletevars;j];
            end;
         end;
      end;
   end;
end;
%
% retrieve variables to keep
%
if isempty(deletevars)==1;
   keepvars = (1:size(x1,2))';
else;
   keepvars = [];
   for i=1:d1;
      done = 0;
      for j=1:size(deletevars,1);
         if i==deletevars(j,1);
            done = 1;
         end;
      end;
      if done==0;
         keepvars = [keepvars;i];
      end;
   end;
   if isempty(x1_r)==0;
      keepvars = keepvars-1;
      deletevars = deletevars-1;
      if size(keepvars,1)>1;
         keepvars = keepvars(2:size(keepvars,1),:);
      else;
         keepvars = [];
      end;
   end;
end;
%
% a second check
%
if size(keepvars,1)>1;
   x = x(:,keepvars');
   d2 = size(x,2);
   for i=2:d2;
      j = d2-i+2;
      x_j = x(:,1:size(x,2)-1);
      if rank(x_j)==rank(x);
         if size(deletevars,1)>0;
            if min(abs(deletevars-j))>0;
               deletevars = [deletevars;j];
            end;
         else;
            deletevars = [deletevars;j];
         end;
      end;
      x = x(:,1:size(x,2)-1);
   end;
   if size(deletevars,1)>0;
      deletevars = sort(deletevars);
      keepvars = [];
      for i=1:d1;
         done = 0;
         for j=1:size(deletevars,1);
            if i==deletevars(j,1);
               done = 1;
            end;
         end;
         if done==0;
            keepvars = [keepvars;i];
         end;
      end;
      if isempty(x1_r)==0;
         keepvars = keepvars-1;
         deletevars = deletevars-1;
         if size(keepvars,1)>1;
            keepvars = keepvars(2:size(keepvars,1),:);
         else;
            keepvars = [];
         end;
      end;
   end;
end;

%
% end of getdelvars.m
%
