%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes multivariate tests for skewness, excess      %
%          kurtosis, and a combination reflecting "normality"    %
%                                                                %
%          Function: normality                                   %
%                                                                %
%          Input:                                                %
%                       epsilon: nxT matrix with residuals       %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                                                                %
%          Output:                                               %
%          		          normalityvec: vector with results        %
%                       df: string with degrees of freedom       %
%                                                                %
%          This version:        January 30, 2006                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [normalityvec,df] = normality(epsilon,t)

[n T] = size(epsilon);

curr_ini = find_inisettings;

%
% Normalizing residuals
%

Omega = (1/T)*epsilon*epsilon';
V = sqrt(diag(diag(Omega)));
V = eye(size(V,1))/V;
C = V*Omega*V;
[H,lambda] = eig(C);
w = H*inverse(sqrt(lambda))*H'*V*epsilon;

%
% An alternative normalization
%
% P = chol(Omega)';
% w = inverse(P)*epsilon;
%

%
% Estimating 3rd and 4th moments
%

b1 = w.*w.*w;
b2 = b1.*w;
b1 = (1/T)*sum(b1')';
b2 = (1/T)*sum(b2')';

%
% Computing test statistics
%

S1 = T*b1'*b1/6;
S2 = T*(b2-3*ones(n,1))'*(b2-3*ones(n,1))/24;
S3 = S1+S2;

if curr_ini.sscorrect==0;
   df = strvcat(num2str(n),num2str(n),num2str(2*n));
   pval = chis_cdf(S1,n);
   normalityvec = [S1 1-pval S2];
   pval = chis_cdf(S2,n);
   normalityvec = [normalityvec 1-pval S3];
   pval = chis_cdf(S3,2*n);
   normalityvec = [normalityvec 1-pval];
else;
   S1 = ((t-1)/(T*n))*S1;
   S2 = ((t-1)/(T*n))*S2;
   S3 = ((t-2)/(T*2*n))*S3;
   df = ['(' num2str(n) ',' num2str(t-1) ')'];
   df = strvcat(df,df);
   df = strvcat(df,['(' num2str(2*n) ',' num2str(t-2) ')']);
   pval = fdis_cdf(S1,n,t-1);
   normalityvec = [S1 1-pval S2];
   pval = fdis_cdf(S2,n,t-1);
   normalityvec = [normalityvec 1-pval S3];
   pval = fdis_cdf(S3,2*n,t-2);
   normalityvec = [normalityvec 1-pval];
end;

%
% end of normality.m
%
