%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing recursive estimates     %
%          of Nyblom under restrictions on beta for I(1)         %
%          models - based on Hansen & Johansen (1999), "Some     %
%          tests for parameter constancy in cointegrated         %
%          VAR-models", Econometrics Journal, 2, 306-333         %
%                                                                %
%          Function: nyblombeta                                  %
%                                                                %
%          Input:                                                %
%                        betar_t: time t estimates of beta       %
%                        betatT: full sample estimates of beta   %
%                        S11_t: time t product moment matrix     %
%                        c_bar: full sample transformation       %
%                           of beta (c_bar = c*inverse(c'*c),    %
%                           where c is an affine function of     %
%                           beta                                 %
%                        c_orth: null space of c                 %
%                        c_bar_orth: null space of c_bar         %
%                        M_T: full sample product moment matrix  %
%                        V_T: full sample variance matrix        %
%                        t: current time period (1,...,T)        %
%                        T: full sample time period (e.g 80)     %
%                                                                %
%          Output:                                               %
%                        result: test-statistic                  %
%                                                                %
%          NOTES: Requires q0=q1=0. Moreover, the short run      %
%                 dynamics are considered fixed at the full      %
%                 sample horizon. Finally, all eigenvalues       %
%                 of the estimated companion matrix must be      %
%                 non-explosive                                  %
%                                                                %
%          This version:        March 24, 2004                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function result = nyblombeta(betar_t,betatT,S11_t,c_bar,c_orth,c_bar_orth,M_T,V_T,t,T)

M_t = c_orth'*S11_t*c_orth;
beta_ct = betar_t*inverse(c_bar'*betar_t);
q_t = c_bar_orth'*(beta_ct-betatT);
result = V_T*q_t'*M_t*inverse(M_T)*M_t*q_t;
result = (t/T)*(t/T)*sum(diag(result));

%
% End of function nyblombeta.m
%
