%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing Ploberger, Krmer and   %
%          Kontrus fluctuation test                              %
%                                                                %
%          Function: pkktest                                     %
%                                                                %
%          Input:                                                %
%                        Z0: nxT matrix                          %
%                        Z1: n+dcoint+q1xT matrix                %
%                        Z2: qxT matrix (may be empty            %
%                        beta: n+dcoint+q1xr matrix              %
%                        d: number of unrestricted deterministic %
%                            variables                           %
%                        dcoint: number of restricted            %
%                            deterministic variables             %
%                        T0: relative time period for starting   %
%                            the recursions                      %
%                        samplebeg: first period (real)          %
%                        sampleend: last period (real)           %
%                        freq: data frequence string (y,q,m)     %
%                        waitbar: indicator if a progress        %
%                            dialog should be displayed or not   %
%                            (default=0, no progress dialog)     %
%                        modelfile: name of model                %
%                            (default is empty string)           %
%                        modelsdir: directory where restrictions %
%                            files are stored                    %
%                            (default is empty string            %
%                        userest: whether to estimate under      %
%                            linear restrictions or not          %
%                            (default=0, no restrictions)        %
%                                                                %
%          Output:                                               %
%                        smat: T-T1xn matrix of fluctuations     %
%                            tests                               %
%                        k:  deg. freedom for fluctuations       %
%                            tests                               %
%                        abort: text string indicating if        %
%                            the iterations have been cancelled  %
%                            or not                              %
%                        dtstr: text string providing the        %
%                            sample dates used in the recursions %
%                                                                %
%          NOTES: requires that Z2 and beta are not both empty   %
%                                                                %
%                 See Ploberger et al (1989), "A New Test for    %
%                 Structural Stability in the Linear Regression  %
%                 Model", Journal of Econometrics, 40, 307-318   %
%                 for details (eq. 9)                            %
%                                                                %
%          This version:        June 16, 2005                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [smat,k,abort,dtstr] = pkktest(Z0,Z1,Z2,beta,d,dcoint,T0,samplebeg,sampleend,freq,waitbar,modelfile,modelsdir,userest,set_dates,real_T,use_c_rest,mm)

if nargin<11;
   waitbar = 0;
end;

if nargin<14;
   modelfile = '';
   modelsdir = '';
   userest = 0;
end;

if nargin<15;
   set_dates = 1;
end;

%
% Preliminaries
%

[n T] = size(Z0);

if nargin<16;
   real_T = T;
end;

if nargin<18;
   use_c_rest = 0;
   mm = [];
end;

if T0<0;
   T1 = round(2*real_T/3);
elseif T0>0.95;
   T1 = round(2*real_T/3);
else;
   T1 = round(real_T*T0);
end;

if isempty(beta)==0;
   r = size(beta,2);
   Z3 = [Z2;beta'*Z1];
else;
   r = 0;
   Z3 = Z2;
end;

k = size(Z3,1);

if T1<=size(Z1,1)+size(Z2,1);
   T1 = size(Z1,1)+size(Z2,1)+1;
   if userest==1;
      T1 = T1+4;
   end;
end;

%
% Initialize
%

smat = zeros(T-T1+1,n);
opened = 0;

abort = '';
dstr = '';

if T1>T;
   if nargout==4;
      dtstr = dstr;
   end;
   return;
end;

if waitbar==1;
   curr_ini = find_inisettings;
   realscrsz = get(0,'ScreenSize');
   input_var.title = ['Progress for ' int2str(T+1-T1) ' recursive estimates'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.label = 'Recursion number:';
   input_var.stop = 0;
   input_var.clock = 0;
   input_var.name = 'Recursive Fluctuation Tests';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','Recursive Fluctuation Tests');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;

%
% Full sample estimates
%

bT = Z0*Z3'*inverse(Z3*Z3');
epsilon = Z0-bT*Z3;
OmegaT = (1/(T-k))*epsilon*epsilon';
ZZchol = chol(Z3*Z3')';

%
% Specify restrictions if asked for (r>0 by construction when userest=1)
% and estimate full sample bT
%

if userest==1;
   file = strcat(modelsdir,'G-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat,rvec] = rmatrix(G,n,r);
   dfa = size(G,1);
   if isempty(Z2)==0;
      Rmat = [zeros(dfa,size(Z2,1)*n) Rmat];
   end;
   [bT,Sigma_b,infomat,dabort] = linrest(Z0,Z3,bT,Rmat,rvec,0);
   epsilon = Z0-bT*Z3;
   OmegaT = (1/(T-k+round(dfa/n)))*epsilon*epsilon';
else;
   dfa = 0;
end;
if use_c_rest==1;
   file = strcat(modelsdir,'C-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(G,n,n);
   [bT,Sigma_b,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0,Z1,Z2,mm.restrict,mm.beta,bT,Rmat_C,rvec_C,mm.p,mm.pe,mm.d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
   epsilon = Z0-bT*Z3;
   OmegaT = (1/(T-k+round((dfa+num_c_rest)/n)))*epsilon*epsilon';
end;

%
% Subsample estimates
%

for t=T1:T;
   if waitbar==1;
      if opened==0;
         drawnow;
         progressdlg([(1/(T-t)) 1]);
         opened = 1;
         start = t;
      else;
         abort = get(mywait,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            progressdlg([(opened/(T+1-start)) opened]);
         end;
      end;
   end;
   T2 = T-t;
   Z0t = Z0(:,1:t);
   Z3t = Z3(:,1:t);
   %
   % Remove any affinely dependent variables from Z3t
   %
   delvar = [];
   if d>0;
      x = Z2(1:d,1:t)';
      if dcoint==1;
         x_ = Z1(size(Z1,1),1:t)';
      else;
         x_ = [];
      end;
      [delvar,keepvar] = getdelvars(x,x_);
      x = checkdeter(x,x_);
      Z3t = [x';Z3t(d+1:k,:)];
      Z2t = [x';Z2(d+1:size(Z2,1),1:t)];
      my_d = size(x',1);
   else;
      if isempty(Z2)==0;
         Z2t = Z2(:,1:t);
      else;
         Z2t = [];
      end;
      my_d = 0;
   end;
   b = Z0t*Z3t'/(Z3t*Z3t');
   if userest==0;
      if use_c_rest==1;
         [b,Sigma_b,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0t,Z1(:,1:t),Z2t,mm.restrict,mm.beta,b,Rmat_C,rvec_C,mm.p,mm.pe,my_d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
      end;
      %
      % Fix b such that full sample estimates are used when
      % a deterministic variable has been removed
      %
      if isempty(delvar)==0;
         d1 = size(delvar,1);
         for i=1:d1;
            dk = delvar(i,1);
            bk = bT(:,dk);
            bs = size(b,2);
            if dk==1;
               b = [bk b];
            elseif dk==k;
               b = [b bk];
            else;
               b = [b(:,1:dk-1) bk b(:,dk:bs)];
            end;
         end;
      end;
   else;
      %
      % Fix Rmat if needed
      %
      Rmatt = Rmat;
      if isempty(delvar)==0;
         d1 = size(delvar,1);
         for i=1:d1;
            dk = delvar(i,1);
            rs = size(Rmatt,2);
            if dk==1;
               Rmatt = Rmatt(:,n+1:rs);
            elseif dk==size(b,2);
               Rmatt = Rmatt(:,1:rs-n);
            else;
               Rmatt = [Rmatt(:,1:(dk-1)*n) Rmatt(:,dk*n+1:rs)];
            end;
         end;
      end;
      %
      % Estimate under linear restrictions if asked for
      %
      [b,Sigma_b,infomat,dabort] = linrest(Z0t,Z3t,b,Rmatt,rvec,0);
      if use_c_rest==1;
         [b,Sigma_b,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0t,Z1(:,1:t),Z2t,mm.restrict,mm.beta,b,Rmat_C,rvec_C,mm.p,mm.pe,my_d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
      end;
      %
      % Fix b such that full sample estimates are used when
      % a deterministic variable has been removed
      %
      if isempty(delvar)==0;
         for i=1:d1;
            dk = delvar(i,1);
            bk = bT(:,dk);
            bs = size(b,2);
            if dk==1;
               b = [bk b];
            elseif dk==k;
               b = [b bk];
            else;
               b = [b(:,1:dk-1) bk b(:,dk:bs)];
            end;
         end;
      end;
   end;
   %
   % Compute S-tests
   %
   for i=1:n;
      if i<=min([size(OmegaT,1) size(b,1) size(bT,1)]);
         s = (t/(T*sqrt(OmegaT(i,i))))*max(abs((b(i,:)-bT(i,:))*ZZchol));
      else;
         s = 0;
      end;
      if i==1;
         smati = s;
      else;
         smati = [smati s];
      end;
   end;
   smat(t-T1+1,:) = smati;
   if set_dates==1;
      dstr = strvcat(dstr,getdate(samplebeg,sampleend,freq,t/T));
   else;
      dstr = '';
   end;
   opened = opened+1;
end;

if waitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

if nargout==4;
   dtstr = dstr;
end;

if userest==1;
   k = k-round(dfa/n);
end;

%
% end of pkktest.m
%
