%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                 %
%          MatLab GUI dialog for displaying graphs of data        %
%          with permanent and transitory components and           %
%          individual shock components                            %
%                                                                 %
%          Function: scdlg                                        %
%                                                                 %
%          Input:                                                 %
%                       selector: case switch function            %
%                       c_mod: structure with model info          %
%          		          y: nxT matrix (n<T) with                  %
%                           endogenous variables                  %
%                       y_perm: nxT matrix with permanent         %
%                           components for the endogenous         %
%                           variables                             %
%                       y_trans: nxT matrix with transitory       %
%                           components for the endogenous         %
%                           variables                             %
%                       perm_shock: n^2xT matrix with permanent   %
%                           shock components                      %
%                       trans_shock: n^2xT matrix with transitory %
%                           shock components                      %
%                       ind: 0-1 indicator if restricted beta     %
%                          is used or not                         %
%                                                                 %
%          Output:                                                %
%          		          h: handle                                 %
%                                                                 %
%          This version:        May 3, 2006                       %
%          Written by:          Anders Warne                      %
%                               Copyright (c) 2001-2006           %
%                                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = scdlg(selector,c_mod,y,y_perm,y_trans,perm_shock,trans_shock,ind)

if strcmp(selector,'init')==1;
   t_m = c_mod;
   if ind==1;
      t_m.ind = '-restr';
   else;
      t_m.ind = '';
   end;
   t_m.y = y;
   t_m.y_perm = y_perm;
   t_m.y_trans = y_trans;
   t_m.perm_shock = perm_shock;
   t_m.trans_shock = trans_shock;
   %
   % setting up the actual sample period
   %
   [t_m.dataindex,t_m.samplebeg,t_m.sampleend] = make_dataindex(t_m.freq,t_m.samplebeg,t_m.sampleend);
   checkyear = floor(t_m.samplebeg);
   checkperiod = t_m.samplebeg-checkyear;
   t_m.subbeginyear = num2str(checkyear);
   if strcmp(t_m.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      t_m.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(t_m.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      t_m.subbeginperiod = num2str(round(checkperiod));
   end;
   %
   % check if t_m.evars is empty and if so generate
   % generic names
   %
   if strcmp(t_m.evars,'')==1;
      for i=1:t_m.n;
         if i==1;
            t_m.evars = 'y_1';
         else;
            t_m.evars = [t_m.evars ' y_' num2str(i)];
         end;
      end;
   end;
   t_m.namevector = vecstr(t_m.evars);
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
else;
   thisexit = 1;
   scgui = findobj('Type','figure','Tag','SCDlg');
   sc = get(scgui,'UserData');
   curr_ini = get(sc.popup,'UserData');
   t_m = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      scgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','scdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-341)/2 (realscrsz(4)-140)/2 341 140], ...
         'Tag','SCDlg', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Shock Decompositions', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','scdlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(scgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 130],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.permcompbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      sc.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Estimation Sample:  ' t_m.subbeginyear ':' t_m.subbeginperiod ' - ' t_m.subendyear ':' t_m.subendperiod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Select Variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');


      sc.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[120 20 80 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',t_m.namevector, ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',1);

      %
      % Create graph button
      %

      sc.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','scdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','Done');

      %
      %  Done button
      %

      sc.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','scdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(scgui,'UserData',sc,...
         'HandleVisibility','callback');

   case 'showgraph'
      series = get(sc.popup,'Value');
      newvalue = series+1;
      if newvalue>t_m.n;
         set(sc.popup,'Value',1);
      else;
         set(sc.popup,'Value',newvalue);
      end;
      if curr_ini.colors==1;
         colorstr = 'b';
         colorstr2 = 'r';
         colorstr3 = [0 1 0];
         typestr = '-';
         typestr2 = '-';
      else;
         colorstr = 'k';
         colorstr2 = [0.5 0.5 0.5];
         colorstr3 = [0.8 0.8 0.8];
         typestr = '--';
         typestr2 = ':';
      end;
      y_i = t_m.y(series,:)';
      y_perm_i = t_m.y_perm(series,:)';
      y_trans_i = t_m.y_trans(series,:)';
      variable = t_m.namevector(series,:);
      variable = strrep(variable,' ','');
      all_plots = [];
      pos = [(realscrsz(3)-612)/2 (realscrsz(4)-484)/2];
      %
      for i=1:t_m.n;
         curr_shock = strtrim(t_m.myshocks(i,:));
         shock_index = series+(t_m.n*(i-1));
         p_shock = t_m.perm_shock(shock_index,:);
         if isempty(t_m.trans_shock)==0;
            t_shock = t_m.trans_shock(shock_index,:);
         else;
            t_shock = [];
         end;
         %
         myplot = figure('Units','pixels', ...
            'Position',[pos 612 484], ...
            'Menubar','none', ...
            'Toolbar','none', ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Name',['Permanent and Transitory Components for ' variable ' - ' curr_shock]);
         %
         all_plots = [all_plots myplot];
         %
         % Fixing position for next graph
         %
         pos = [pos(1,1)+22 pos(1,2)-22];
         if pos(1,1)>(realscrsz(3)-612);
            pos = [0 realscrsz(4)-554];
         end;
         if pos(1,2)>(realscrsz(4)-554);
            pos = [0 realscrsz(4)-554];
         elseif pos(1,2)<32;
            pos = [0 realscrsz(4)-554];
         end;
         %
         % default figure properties
         %
         setfigureproperties(myplot);
         %
         %
         % Setting up the plots
         %
         if isempty(t_shock)==0;
            subplot(2,1,1);
         end;
         %
         % check if we should create a zero-line
         %
         if ((max(y_i)>0)|(max(y_perm_i)>0)|(max(p_shock)>0))&((min(y_i)<0)|(min(y_perm_i)<0)|(min(p_shock)<0));
            plot([min(t_m.dataindex) max(t_m.dataindex)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
            hold('on');
         end;
         line_one = plot(t_m.dataindex,y_i,'-','LineWidth',1,'Color',colorstr,'Tag',variable);
         hold('on');
         line_two = plot(t_m.dataindex,y_perm_i,typestr,'LineWidth',1,'Color',colorstr2,'Tag',['pc-' variable]);
         hold('on');
         line_three = plot(t_m.dataindex,p_shock,typestr2,'LineWidth',1,'Color',colorstr3,'Tag',['pc-' curr_shock '-' variable]);
         my_pos = 1;
         if (y_i(t_m.T,1))>(y_i(1,1));
            my_pos = 2;
         end;
         set(gca,'XLim',[t_m.dataindex(1,1) t_m.dataindex(size(t_m.dataindex,1),1)]);
         %
         if istexgreek(curr_shock)==1;
            this_shock = ['\' curr_shock];
         else;
            this_shock = curr_shock;
         end;
         %
         setaxesfonts('Title',['Actual, Permanent, and Permanent ' this_shock ' Components for ' variable ]);
         myleg = legend_fcn([line_one line_two line_three],variable,[variable '^{P}'],[variable '^{P} (' this_shock ')'],my_pos);
         setaxesfonts(myleg);
         set(gca,'UserData',myleg);
         axeshandles = gca;
         hold('off');
         %
         % transitory component and transitory shock component
         %
         if isempty(t_shock)==0;
            subplot(2,1,2);
            line_one = plot(t_m.dataindex,y_trans_i,'-','LineWidth',1,'Color',colorstr,'Tag',['tc-' variable]);
            hold('on');
            line_two = plot(t_m.dataindex,t_shock,typestr,'LineWidth',1,'Color',colorstr2,'Tag',['tc-' curr_shock '-' variable]);
            hold('on');
            plot(t_m.dataindex,zeros(t_m.T,1),'-','LineWidth',1,'Color','k');
            set(gca,'XLim',[t_m.dataindex(1,1) t_m.dataindex(size(t_m.dataindex,1),1)]);
            %
            setaxesfonts('Title',['Transitory and Transitory ' this_shock ' Components for ' variable]);
            myleg = legend_fcn([line_one line_two],[variable '^{Trans}'],[variable '^{Trans} (' this_shock ')']);
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            axeshandles = [axeshandles gca];
         end;
         toolbar_fcn(axeshandles);
         hold('off');
      end;
      %
      if curr_ini.external==0;
         set(all_plots,'Visible','on');
      else;
         %
         % all figures need to be saved and then opened
         %
         files = '';
         stats = [];
         for i=1:t_m.n;
            curr_shock = strtrim(t_m.myshocks(i,:));
            file = [t_m.outputdir 'pc-tc-' variable '-' curr_shock t_m.ind '-rank-' num2str(t_m.r)];
            files = strvcat(files,file);
            stats = [stats autosavefigure(all_plots(1,i),file)];
         end;
         for i=1:t_m.n;
            if stats(1,i)==0;
               autoshowfigure(strtrim(files(i,:)));
            end;
         end;
      end;
      set(scgui,'UserData',sc);
   case 'done'
      set(sc.done,'UserData','done');
end;

if nargout==1;
   h = scgui;
end;

if strcmp(selector,'done')==0;
   curr_ini.models = t_m;
   set(sc.popup,'UserData',curr_ini);
   set(scgui,'UserData',sc);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(scgui,'Name') ' ' fix_filepath([pwd '\icons\permcomp.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(sc.icon,'UserData',pid);
      end;
   end;
   %
   set(scgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(scgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(sc.done);
   end;
   waitfor(sc.done,'UserData','done');
end;

%
% end of scdlg.m
%
